/**************************************************************************
 *	Copyright (C) 2004  iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, 
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include "gl_util.hpp"

#ifdef _DEBUG
long totalTime = 0;
#endif // _DEBUG

#include <iostream>

using namespace std;

TTF_Font*  GLUtil::openFont(string fontname, int point)
{
	TTF_Font* font = TTF_OpenFont(fontname.c_str(), point);

	if(!font)
	{
		cout<<"ERROR : "<<TTF_GetError()<<endl;
	}

	return font;
}

SDL_Surface* GLUtil::createTextSurfaceGL(int number)
{
	SDL_Surface* surf = NULL;

	char* tmp = (char*) malloc(sizeof(char)*15);
	sprintf(tmp, "%d", number);
	surf = createTextSurfaceGL(tmp);
	free(tmp);
	return surf;
}

SDL_Surface* GLUtil::createTextSurfaceGL(const char* text )
{
	return createTextSurfaceGL(text, 64, 32, 25 , TRUE);
}

SDL_Surface* GLUtil::createTextSurfaceGL(const char* text, int w, int h, int point, bool xCenter )
{

	Uint32 rmask, gmask, bmask, amask, bkcolor;
	#if SDL_BYTEORDER == SDL_BIG_ENDIAN
    	rmask = 0xff000000;
    	gmask = 0x00ff0000;
    	bmask = 0x0000ff00;
    	amask = 0x000000ff;
	bkcolor = 0xFFFFFF00;
	#else
    	rmask = 0x000000ff;
    	gmask = 0x0000ff00;
    	bmask = 0x00ff0000;
    	amask = 0xff000000;
	bkcolor = 0x00ffffff;
	#endif

    SDL_Surface *image1 = NULL;
	SDL_Surface *surfText = NULL;

	SDL_Rect destrect;

	destrect.h = 0;
	destrect.w = 0;

	image1 = SDL_CreateRGBSurface(SDL_SWSURFACE, w, h, 32,rmask, gmask, bmask, amask);

	if((image1 == NULL) || (text == NULL))
		return 0;
	
	SDL_Color black = { 0x00, 0x00, 0x00, 0 };
//	SDL_Color white = { 0xFF, 0xFF, 0xFF, 0 };
	
	TTF_Font *font;

	// This is such a stupid/ugly hack
	if(point == 25)
	{
		if(!font25)
			font25 = openFont(m_FontFile, 25);

		font = font25;

	}
	else if (point == 40)
	{
		if(!font64)
			font64 = TTF_OpenFont(m_FontFile.c_str(), 40);
//			openFont(m_FontFile, 40, font64);

		font = font64;
	}
	else if (point == 30)
	{
		if(!font30)
			font30 = TTF_OpenFont(m_FontFile.c_str(), 30);
//			openFont(m_FontFile, 30, font30);

		font = font30;

	}
	else 
	{
		font = TTF_OpenFont(m_FontFile.c_str(), point);
	}

	if ( font == NULL ) {
		return 0;
	}


	SDL_FillRect(image1, NULL, bkcolor);
	
	if(strlen(text) > 0)
	{
		surfText = TTF_RenderText_Solid(font, text, black);

		if(!surfText)
		{
			cout<<"Error, this shouldn't be NULL"<<endl;
			return NULL;
		}
		
		if(xCenter)
			destrect.x = ((image1->w - surfText->clip_rect.w)/2);
		else
			destrect.x = 0;
		
		destrect.y = ((image1->h - surfText->clip_rect.h)/2);
		SDL_BlitSurface(surfText, NULL , image1, &destrect);

	}

	SDL_FreeSurface(surfText);
	return image1;
}

inline void invertRow(Uint8* row, int size)
{
	Uint8 tmp = 0;

	Uint8* start = row;
	Uint8* end = (row + size - 1);

	for(int i = 0; i < size/2; ++i)
	{
		tmp = (*start);
		(*start++) = *end;
		(*end--) = tmp;
	}
}

SDL_Surface* GLUtil::LoadBMP(const char *filename, bool hor, bool ver)
{
    SDL_Surface *image;

    image = SDL_LoadBMP(filename);

    if ( image == NULL ) {
        fprintf(stderr, "Unable to load %s: %s\n", filename, SDL_GetError());
        return(NULL);
    }

    invertRow((Uint8*)image->pixels, image->h * image->pitch);

    return(image);
}

GLUtil::~GLUtil()
{
}
