/**************************************************************************
 *	Copyright (C) 2004  iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, 
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include "host-draw.hpp"

GLfloat sphereVal[816];
GLfloat normalsVal[816];
GLfloat texturesVal[544];

GLfloat* tSphere;
GLfloat* ipLabel;
GLfloat* vaRouter;

bool inited = false;
bool tInited = false;
bool ipIni	= false;
bool vaiRouter	= false;

void cleanUp();
/*********************
	Drawing
**********************/
void HostDraw::drawTower()
{
	int n = (int)m_Ports.size();

	if(m_vaLines == NULL)
	{
		m_vaLines = new GLfloat[n * 6];
		cout<<"n = "<<n<<endl;
		int s = 0;
		for(int i = 1; i <= n; ++i)
		{
			*(m_vaLines + s + 0) = 0.0f;
			*(m_vaLines + s + 1) = 0.0f  - (0.045f)*i;
			*(m_vaLines + s + 2) = 0.0f;

			*(m_vaLines + s + 3) = 0.0f;
			*(m_vaLines + s + 4) = 0.04f - (0.045f)*i;
			*(m_vaLines + s + 5) = 0.07f;
			s += 6;
		}
	}

	glPushMatrix();
		glColor3f(1.0f, 0.0f,0.0f);
		glDisable(GL_LIGHTING);
		glBegin(GL_LINES);
			glVertex3f( 0, 0, 0);
			glVertex3f( 0, -(0.1f +(0.045f)*n),0.0f);
		glEnd();
		glEnable(GL_LIGHTING);

		glTranslatef(0,-0.1f,0.0f);

	for(vector<PortDraw*>::iterator i = m_Ports.begin(); i != m_Ports.end(); ++i)
	{
		glTranslatef(0,-0.045f,0.0f);
		(*i)->draw(GL_RENDER);
	}

	glPopMatrix();
	glPushMatrix();
	glTranslatef(0,-0.099f,0.0f);
	glInterleavedArrays(GL_V3F, 0, m_vaLines);
	glDrawArrays(GL_LINES, 0, 2*n);
	glColor3f(1.0f, 0.0f,0.0f);
	glInterleavedArrays(GL_N3F_V3F, 0, tSphere);

#ifdef GL_EXT_compiled_vertex_array	
	if(m_Config->enableCVA)
	 	glLockArraysEXT(0, 272);	
#endif	
	for(int i = 0; i < n; ++i)
	{
		glTranslatef(0,-0.045f,0.0f);
		glDrawArrays(GL_QUAD_STRIP, 0, 272);
	}
#ifdef GL_EXT_compiled_vertex_array
	if(m_Config->enableCVA)
		glUnlockArraysEXT();
#endif

	glPopMatrix();
}

void HostDraw::drawPorts()
{
	drawTower();	
}

void HostDraw::draw(GLenum mode)
{
	generate();
}

void HostDraw::generate()
{
	m_Dirty = FALSE;

	glPushMatrix();
	glRotatef(90,0,1.0f,0);
	glEnable(GL_TEXTURE_2D);

	glBindTexture(GL_TEXTURE_2D, m_TexStore->getOSTexture(m_OSFamily));
	
	if(this->m_HostType == "general purpose")
	{
		drawHostGeneric();
	}
	else if(this->m_HostType == "router")
	{
		drawHostRouter();
	}
	else
	{
		drawSphere(.05f);
	}
	glPopMatrix();

	m_Radius = 0.4f;

	glPushMatrix();
	drawPorts();
	drawIP();

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, texIp);
	glInterleavedArrays(GL_T2F_N3F_V3F, 0, ipLabel);

	glPushMatrix();
		glTranslatef(0.0f, -0.06f, -0.04f);
		glRotatef(45, 0.0f, -1.0f, 0.0f);
		glDrawArrays(GL_QUAD_STRIP, 0, 30);
	glPopMatrix();

	glPopMatrix();
	glDisable(GL_TEXTURE_2D);
}



/*********************
	Construction
**********************/
HostDraw::HostDraw()
{
	m_Dirty = TRUE;
	m_vaLines = NULL;
}

/*********************
	Destruction
**********************/
HostDraw::~HostDraw()
{
	PortDraw* port = NULL;
	//glDeleteLists(m_dList, 1);

	for(vector<PortDraw*>::iterator i = m_Ports.begin(); i != m_Ports.end(); ++i)
	{
		delete port;
	}

	m_Ports.clear();

	// Destroy Textures;
//	glDeleteTextures(6, &texOs[0]);
	glDeleteTextures(1, &texFlag);
	glDeleteTextures(1, &texIp);
	glDeleteTextures(1, &texHost);
	cleanUp();
}


//TODO: Make this use the texture cache?
void HostDraw::setIp(const char* ip)
{
	if(ip == NULL)
		return;

	SDL_Surface* tmpSurf = NULL;

	m_ipAddress = ip;

	if(m_Config == NULL)
	{
		printf("HD::setIp -) ERROR m_Config is NULL\n");
		return;
	}
	if(m_Config->m_glUtil == NULL)
	{
		printf("HD::setIp -) ERROR m_Config->m_glUtil is NULL\n");
		return;	
	}

	tmpSurf = m_Config->m_glUtil->createTextSurfaceGL(m_ipAddress.c_str(), 256, 32, 30, TRUE);

	if(tmpSurf == NULL)
		return;

	glGenTextures(1, &texIp);

    glBindTexture(GL_TEXTURE_2D, texIp);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR); // scale linearly when image bigger than texture
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR); // scale linearly when image smalled than texture

    glTexImage2D(GL_TEXTURE_2D, 0, 3, tmpSurf->w, tmpSurf->h, 0, GL_RGBA, GL_UNSIGNED_BYTE, tmpSurf->pixels);
	SDL_FreeSurface(tmpSurf);

}
//TODO: Change this to just generate the geometry, possibly in a static fashion
//and have the ipIni test happen in the actual drawing loop upstairs
void HostDraw::drawIP()
{
	if(!ipIni)
	{
		const float labelWidth = 0.03f;
		const float rad = 0.16f;
		const int	n = 15;
		const int slice = 90/n;

		float x = 0;
		float z = 0;
		float mag = 0;
		int s = 0;
	
		ipLabel = new GLfloat[16*n];
		for(float j = 0; j < n ; ++j)
		{
			x=rad*cos((float)((Pi/180)*j*slice));
			z=rad*sin((float)((Pi/180)*j*slice));
			mag = sqrt(x*x + z*z);

			*(ipLabel + s++) = (float)(1 - j/n);
			*(ipLabel + s++) = 1.0f;
			*(ipLabel + s++) = (float)x/mag;
			*(ipLabel + s++) = 0.0f;
			*(ipLabel + s++) = (float)z/mag;
			*(ipLabel + s++) = (float)x;
			*(ipLabel + s++) = 0.0f;
			*(ipLabel + s++) = (float)z;

			*(ipLabel + s++) = (float)(1 - j/n);
			*(ipLabel + s++) = 0.0f;
			*(ipLabel + s++) = (float)x/mag;
			*(ipLabel + s++) = 0.0f;
			*(ipLabel + s++) = (float)z/mag;
			*(ipLabel + s++) = (float)x;
			*(ipLabel + s++) = labelWidth;
			*(ipLabel + s++) = (float)z;

		}
		ipIni = true;
	}
}

//TODO: Change this to use packed data instead
void drawHostSphere()
{
	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_NORMAL_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);

	if(!inited)
	{
		int sI = 0;
		int sT = 0;
		int sN = 0;

		float r = 0.1f;
		int n = 16;
		int i,j;
		float theta1,theta2,theta3;
		xyz e,p,c;

		//if(port > 0)
		//	glPrint("%i", port);

		c.x = 0;
		c.z = 0;
		c.y = 0;

		if (r < 0)
			r = -r;
		if (n < 0)
			n = -n;
		if (n < 4 || r <= 0) {
			glBegin(GL_POINTS);
			glVertex3f(c.x,c.y,c.z);
			glEnd();
			return;
		}
//	int Count = 0;

   for (j=0; j<n/2; ++j) 
   {
      theta1 = j * TWOPI / n - PID2;
      theta2 = (j + 1) * TWOPI / n - PID2;

      for (i=0;i<=n;i++) {
         theta3 = i * TWOPI / n;

        e.x = cos(theta2) * cos(theta3);
        e.y = sin(theta2);
        e.z = cos(theta2) * sin(theta3);
        p.x = c.x + r * e.x;
        p.y = c.y + r * e.y;
        p.z = c.z + r * e.z;

		normalsVal[sN++] = e.x;
		normalsVal[sN++] = e.y;
		normalsVal[sN++] = e.z;
		texturesVal[sT++] = i/(float)n;
		texturesVal[sT++] = 2*(j+1)/(float)n;

		sphereVal[sI++] = p.x;
		sphereVal[sI++] = p.y;
		sphereVal[sI++] = p.z;

         e.x = cos(theta1) * cos(theta3);
         e.y = sin(theta1);
         e.z = cos(theta1) * sin(theta3);
         p.x = c.x + r * e.x;
         p.y = c.y + r * e.y;
         p.z = c.z + r * e.z;

 		normalsVal[sN++] = e.x;
		normalsVal[sN++] = e.y;
		normalsVal[sN++] = e.z;
		texturesVal[sT++] = i/(float)n;
		texturesVal[sT++] = 2*j/(float)n;

		sphereVal[sI++] = p.x;
		sphereVal[sI++] = p.y;
		sphereVal[sI++] = p.z;

      }
	 } // for (j=0; j<n/2; ++j)
		inited = TRUE;
	} // if(!inited)

	glTexCoordPointer(2, GL_FLOAT, 0, texturesVal);
	glNormalPointer(GL_FLOAT, 0, normalsVal);
	glVertexPointer(3, GL_FLOAT, 0, sphereVal);
	
//	if(m_Config->enableCVA)
//		glLockArraysEXT(0, 272);

	glDrawArrays(GL_QUAD_STRIP, 0, 272);
	
//	if(m_Config->enableCVA)
//		glUnlockArraysEXT();

	glDisableClientState(GL_VERTEX_ARRAY);
	glDisableClientState(GL_NORMAL_ARRAY);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);

}

void drawTinySphere(bool generate = FALSE)
{
	if(!tInited)
	{
		int s = 0;

		float r = 0.009f;
		int n = 16;
		int i,j;
		float theta1,theta2,theta3;
		xyz e,p,c;

		int sz = (n+1)*n*3;

		tSphere		= new GLfloat[2*sz];

		c.x = 0;
		c.z = 0;
		c.y = 0;

		if (n < 4 || r <= 0) {
			glBegin(GL_POINTS);
			glVertex3f(c.x,c.y,c.z);
			glEnd();
			return;
		}

		for (j=0; j<n/2; ++j) 
		{
			theta1 = j * TWOPI / n - PID2;
			theta2 = (j + 1) * TWOPI / n - PID2;

		    for (i=0;i<=n;i++) 
			{
				theta3 = i * TWOPI / n;

				//TODO: bring cos/sin(t1,t2) out of this loop
				// call cos/sin(t3) only once per iteration
		        e.x = cos(theta2) * cos(theta3);
			    e.y = sin(theta2);
				e.z = cos(theta2) * sin(theta3);
				p.x = c.x + r * e.x;
				p.y = c.y + r * e.y;
				p.z = c.z + r * e.z;

				*(tSphere + s++) = e.x;
				*(tSphere + s++) = e.y;
				*(tSphere + s++) = e.z;
				*(tSphere + s++) = p.x;
				*(tSphere + s++) = p.y;
				*(tSphere + s++) = p.z;

		        e.x = cos(theta1) * cos(theta3);
				e.y = sin(theta1);
				e.z = cos(theta1) * sin(theta3);
				p.x = c.x + r * e.x;
				p.y = c.y + r * e.y;
				p.z = c.z + r * e.z;

				*(tSphere + s++) = e.x;
				*(tSphere + s++) = e.y;
				*(tSphere + s++) = e.z;
				*(tSphere + s++) = p.x;
				*(tSphere + s++) = p.y;
				*(tSphere + s++) = p.z;

			}
		} // for (j=0; j<n/2; ++j)
		tInited = TRUE;

	} // if(!inited)
}

// TODO: Convert this to use packed Vertex Arrays
// TODO: recalculate so that transforms are not done
void HostDraw::drawHostGeneric()
{
	glPushMatrix();
	glRotatef(-90, 0.0f, 1.0f, 0.0f);
	glRotatef(90, 0.0f, 0.0f, 1.0f);
	glRotatef(15, 0.0f, 1.0f, 0.0f);
	
	glTranslatef(-0.04f, -0.05f,0.05f);
	glBegin(GL_QUADS);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(0.0f, 0.0f, 0.0f);
		glNormal3f(0.0f, 0.0f, 1.0f);
		glTexCoord2f(0.0f, 1.0f);
		glVertex3f(0.1f, 0.0f, 0.0f);
		glNormal3f(0.0f, 0.0f, 1.0f);
		glTexCoord2f(1.0f, 1.0f);
		glVertex3f(0.1f, 0.1f, 0.0f);
		glNormal3f(0.0f, 0.0f, 1.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex3f(0.0f, 0.1f, 0.0f);
		glNormal3f(0.0f, 0.0f, 1.0f);
	glEnd();
	glPopMatrix();
}


void HostDraw::drawHostRouter()
{
	const int n = 90;

	if(!vaiRouter)
	{     
		float x, y;
		const float labelWidth = 0.06f;
		const float m_Radius = 0.06f;
		float i = 0;
		
		int s = 0;	
		float step = 360/n;
		float mag = 0;

		vaRouter = new GLfloat[16*(n + 1)];
	
		for(float j = 0; j <= n; ++j)
		{
			x=m_Radius*cos((float)((Pi/180)*j*step));
			y=m_Radius*sin((float)((Pi/180)*j*step));

			mag = sqrt(x*x + y*y);

			if(mag <= 0)
				mag = 1; 

 			// Textures;
			if((j > 120/step) && (j <= 240/step))
			{
				*(vaRouter + s) = (float)(i*step/(120));
                		*(vaRouter + s + 9) = 1.0f;
				++i;
			}
			else	
			{
				*(vaRouter + s) = 0.0f;
			}

			*(vaRouter + s + 1) = 0.0f;
            *(vaRouter + s + 8) = *(vaRouter + s);

			// Normals	
			*(vaRouter + s + 2) = x/mag;
            *(vaRouter + s + 3) = 0.0f;
            *(vaRouter + s + 4) = y/mag;

            *(vaRouter + s + 10) = *(vaRouter + s + 2);
            *(vaRouter + s + 11) = *(vaRouter + s + 3);
            *(vaRouter + s + 12) = *(vaRouter + s + 4);

			// Verteces
            *(vaRouter + s + 5) = x;
            *(vaRouter + s + 6) = 0.0f;
            *(vaRouter + s + 7) = y;
            *(vaRouter + s + 13) = x; 
            *(vaRouter + s + 14) = labelWidth;
            *(vaRouter + s + 15) = y;
		
			s += 16;
		}
		vaiRouter = true;
	}

	glInterleavedArrays(GL_T2F_N3F_V3F, 0, vaRouter);
	glDrawArrays(GL_QUAD_STRIP, 0, 2*(n + 1));

}

int HostDraw::getPortCount()
{
	return (int) m_Ports.size();
}

void HostDraw::addPort(PortDraw* port)
{
	if(port != NULL)
		m_Ports.push_back(port);
}

void HostDraw::addName(string* name)
{
	m_Hostnames.push_back(name);	
}

// this is only here temporarily untill we get an object cache going
void cleanUp()
{

	if(ipIni)
		delete[] ipLabel;
	// delete[] hSphere;
	if(tInited)
		delete[] tSphere;
	if(vaiRouter)		
		delete[] vaRouter;

	ipIni = tInited = vaiRouter = false;
}

