/**************************************************************************
 *	Copyright (C) 2004  iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, 
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include "xml_cfg.hpp"

/*********************
	   Parsing
**********************/
int XMLConfig::initParser()
{
	try
	{
		XERCES_CPP_NAMESPACE_QUALIFIER XMLPlatformUtils::Initialize();
	}
	catch(const XERCES_CPP_NAMESPACE_QUALIFIER XMLException& toCatch)
	{
		char* message = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(toCatch.getMessage());
		cout << "Error during initialization! :\n"
		     << message << "\n";

		XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&message);
		return FALSE;
	}

	m_Parser = new XERCES_CPP_NAMESPACE_QUALIFIER XercesDOMParser();
	m_Parser->setValidationScheme(XercesDOMParser::Val_Always);    // optional.
	m_Parser->setDoNamespaces(true);    // optional
	m_Parser->setErrorHandler(m_errHandler);

	return TRUE;

}


bool XMLConfig::parse(string file)
{

	try
	{
		m_Parser->resetDocumentPool();
		m_Parser->parse(file.c_str());
	}
	catch (const XERCES_CPP_NAMESPACE_QUALIFIER XMLException& toCatch)
	{
		char* message = XMLString::transcode(toCatch.getMessage());
		cout << "Exception message is: \n"
		     << message << "\n";
		XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&message);
		return FALSE;
	}
	catch (const DOMException& toCatch)
	{
		char* message = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(toCatch.msg);
		cout << "Exception message is: \n"
		     << message << "\n";
		XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&message);
		return FALSE;
	}
	catch (...)
	{
		cout << "XMLConfig::parse() -> Unexpected Exception parsing file: "<<file<<endl ;
		return FALSE;
	}

	inited = TRUE;
	return TRUE;
}

/*********************
  Construction
**********************/
XMLConfig::XMLConfig()
{ 
	m_TexStore	= new TextureStore();
	m_TreePorts	= FALSE;
	m_FamIndex	= 0;
	m_DataDir	= FE_DATA_DIR;

	//m_OSInfoFile	= m_DataDir + "os_info.xml"; 
	m_ConfigFile	= m_DataDir + "fe_config.xml"; 
	m_errHandler	= (ErrorHandler*) new HandlerBase();
	m_glUtil	= NULL;
	m_Parser	= NULL;

	m_ResW	= 640;
	m_ResH	= 480;

	enableEarth = TRUE;
	inited= FALSE;
};

/*********************
   Destruction
**********************/
XMLConfig::~XMLConfig()
{
  	while(!m_OStoFamily.empty())
	{
		m_OStoFamily.erase(m_OStoFamily.begin());
    	}

//	delete m_glUtil;
	delete m_errHandler;
	
	if(inited)
		delete m_Parser;

	delete m_TexStore;
	XERCES_CPP_NAMESPACE_QUALIFIER XMLPlatformUtils::Terminate();
}

/*********************
   Element Parsing
**********************/
bool XMLConfig::parseOSInfo(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode* node)
{
	XERCES_CPP_NAMESPACE_QUALIFIER DOMNode* xmlSubNode = NULL;
	XERCES_CPP_NAMESPACE_QUALIFIER DOMNamedNodeMap* xmlAtt = NULL;
	XMLCh* tmpCh = NULL;

	char *sname = NULL;
	xmlAtt = node->getAttributes();

	if(xmlAtt != NULL)
	{
		tmpCh = XMLString::transcode("xml_file");
		xmlSubNode = xmlAtt->getNamedItem(tmpCh);
		if(xmlSubNode)
		{
			sname = XMLString::transcode(xmlSubNode->getNodeValue());
			m_OSInfoFile = m_DataDir + sname;
			delete sname;
		}
		delete tmpCh;
	}

	return TRUE;	
}

bool XMLConfig::parseScreenshot(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode* node)
{
	XERCES_CPP_NAMESPACE_QUALIFIER DOMNode* xmlSubNode = NULL;
	XERCES_CPP_NAMESPACE_QUALIFIER DOMNamedNodeMap* xmlAtt = NULL;
	XMLCh* tmpCh = NULL;

	char *sname = NULL;
	xmlAtt = node->getAttributes();

	if(xmlAtt != NULL)
	{
		tmpCh = XMLString::transcode("ver-title");
		xmlSubNode = xmlAtt->getNamedItem(tmpCh);
		if(xmlSubNode)
		{
			sname = XMLString::transcode(xmlSubNode->getNodeValue());
			// Enter other code here
			delete sname;
		}
		delete tmpCh;
	}
	
	return TRUE;	
}

bool XMLConfig::parseDisplay(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode* node)
{
	
	
	return TRUE;	
}

bool XMLConfig::parseGraphics(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode* node)
{

	XERCES_CPP_NAMESPACE_QUALIFIER DOMNode*		xmlSubNode	= NULL;
	XERCES_CPP_NAMESPACE_QUALIFIER DOMNamedNodeMap*	xmlAtt		= NULL;
	XERCES_CPP_NAMESPACE_QUALIFIER DOMNode*		childNode	= NULL;
	
	char	*sname	= NULL;
	XMLCh*	tmpCh	= NULL;

	childNode = node->getFirstChild();

	while(childNode)
	{
		sname = XMLString::transcode(childNode->getNodeName());

		if(strcmp(sname, "texture-font")==0)
		{
			xmlAtt = childNode->getAttributes();

			if(xmlAtt != NULL)
			{
				tmpCh = XMLString::transcode("default");
				xmlSubNode = xmlAtt->getNamedItem(tmpCh);

				if(xmlSubNode)
				{
					delete sname;
					sname = XMLString::transcode(xmlSubNode->getNodeValue());
					m_FontFile = m_DataDir + sname;
				}

				delete tmpCh;
			}
		}
		else if(strcmp(sname, "opengl")==0)
		{
			xmlAtt = childNode->getAttributes();
	        if(xmlAtt != NULL)
            {
				tmpCh = XMLString::transcode("vbo");
				xmlSubNode = xmlAtt->getNamedItem(tmpCh);
     			if(xmlSubNode)
				{
                    delete sname;
					sname = XMLString::transcode(xmlSubNode->getNodeValue());

					if(strcmp(sname, "true")==0)
						enableVBO = true;
				}
				delete tmpCh;
			}
		}
		else if(strcmp(sname, "resolution")==0)
		{
                       xmlAtt = childNode->getAttributes();

                        if(xmlAtt != NULL)
                        {
                                tmpCh = XMLString::transcode("w");
                                xmlSubNode = xmlAtt->getNamedItem(tmpCh);

                                if(xmlSubNode)
                                {
                                        delete sname;
                                        sname = XMLString::transcode(xmlSubNode->getNodeValue());
                                       	m_ResW = atoi(sname); 
                                }
								delete tmpCh;

                                tmpCh = XMLString::transcode("h");
                                xmlSubNode = xmlAtt->getNamedItem(tmpCh);

                                if(xmlSubNode)
                                {
                                        delete sname;
                                        sname = XMLString::transcode(xmlSubNode->getNodeValue());
                                       	m_ResH = atoi(sname); 
                                } 
								delete tmpCh;
                        }
  			
		}
		else if(strcmp(sname, "root_node")==0)
		{
                    	xmlAtt = childNode->getAttributes();

                        if(xmlAtt != NULL)
                        {
                                tmpCh = XMLString::transcode("tex_enabled");
                                xmlSubNode = xmlAtt->getNamedItem(tmpCh);

                                if(xmlSubNode)
                                {
                                        delete sname;
                                        sname = XMLString::transcode(xmlSubNode->getNodeValue());
                               		
										if(strcmp("true", sname) == 0)
											enableEarth = TRUE;
										else
											enableEarth = FALSE;

								}
                                delete tmpCh;

                                tmpCh = XMLString::transcode("axial_tilt");
                                xmlSubNode = xmlAtt->getNamedItem(tmpCh);

                                if(xmlSubNode)
                                {
                                    delete sname;
                                    sname = XMLString::transcode(xmlSubNode->getNodeValue());
                                    m_RootTilt = (float)atof(sname);
                                }
                                delete tmpCh;
                                
								tmpCh = XMLString::transcode("texture");
                                xmlSubNode = xmlAtt->getNamedItem(tmpCh);

                                if(xmlSubNode)
                                {
                                    delete sname;
                                    sname = XMLString::transcode(xmlSubNode->getNodeValue());
									m_RootTex = m_DataDir + sname;
                                }
                                delete tmpCh;
                        }
	
		}

		delete sname;

		childNode = childNode->getNextSibling();
	}

	return TRUE;
}

bool XMLConfig::parseAudio(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode* node)
{
	
	
	return TRUE;	
}

/*********************
	   Settings
**********************/
bool XMLConfig::loadSettings()
{
	if(parse(m_ConfigFile))
        return this->parseSettings();
	else
		return FALSE;
}

bool XMLConfig::saveSettings()
{

	return false;
}

bool XMLConfig::parseSettings()
{
	XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument*	doc		= NULL;
	//XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument*	mode	= NULL;
	XERCES_CPP_NAMESPACE_QUALIFIER DOMNodeList*	nodes	= NULL;


	XMLCh* tmpCh = NULL;

	//char *sname = NULL;

	if(m_Parser != NULL)
	{
		doc = m_Parser->getDocument();

		if(doc == NULL)
			return FALSE;
		
		tmpCh = XMLString::transcode("os-info");
		nodes  = doc->getElementsByTagName(tmpCh);
		delete tmpCh;

	if(nodes->getLength() > 0)
            if(!parseOSInfo(nodes->item(0)))
				return FALSE;

		tmpCh = XMLString::transcode("display");
		nodes  = doc->getElementsByTagName(tmpCh);
		delete tmpCh;

	if(nodes->getLength() > 0)
            if(!parseDisplay(nodes->item(0)))
				return FALSE;

		tmpCh = XMLString::transcode("screenshot");
		nodes  = doc->getElementsByTagName(tmpCh);
		delete tmpCh;

	if(nodes->getLength() > 0)
            if(!parseScreenshot(nodes->item(0)))
				return FALSE;

		tmpCh = XMLString::transcode("graphics");
		nodes  = doc->getElementsByTagName(tmpCh);
		delete tmpCh;

        if(nodes->getLength() > 0)
            if(!parseGraphics(nodes->item(0)))
				return FALSE;

	}

	return TRUE;
}

/*********************
	   OS
**********************/
bool XMLConfig::loadOSInfo()
{
	if(parse(m_OSInfoFile))
        return this->parseOSInfo();
	else
		return FALSE;
}


// Parses os_info.xml
bool XMLConfig::parseOSInfo()
{
	XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument*	doc = NULL;
	XERCES_CPP_NAMESPACE_QUALIFIER DOMNodeList*	nodesOSFam = NULL;
	XERCES_CPP_NAMESPACE_QUALIFIER DOMNodeList*	nodesOSFamTex = NULL;
	XERCES_CPP_NAMESPACE_QUALIFIER DOMNode*		node = NULL;
	XERCES_CPP_NAMESPACE_QUALIFIER DOMNode*		xmlSubNode = NULL;
	XERCES_CPP_NAMESPACE_QUALIFIER DOMNamedNodeMap* xmlAtt = NULL;
	XMLCh* tmpCh = NULL;
	XMLSize_t i = 0;

	char *sname = NULL;
	OS_FAMILY::OS osFam = OS_FAMILY::UNKNOWN;


	if(m_Parser != NULL)
	{
		doc = m_Parser->getDocument();

		if(doc == NULL)
			return FALSE;
		
		tmpCh = XMLString::transcode("family");
		nodesOSFam  = doc->getElementsByTagName(tmpCh);
		delete tmpCh;
		
		tmpCh = XMLString::transcode("texture");
		nodesOSFamTex  = doc->getElementsByTagName(tmpCh);
		delete tmpCh;

		if((nodesOSFam == NULL) || (nodesOSFamTex == NULL))
			return FALSE;

		// Firstly get all the node families
		for(i = 0; i < nodesOSFam->getLength(); ++i)
		{
			node = nodesOSFam->item(i);
			xmlAtt = node->getAttributes();
/*			
			tmpCh = XMLString::transcode("family_id");
			xmlSubNode = xmlAtt->getNamedItem(tmpCh);
			sname = XMLString::transcode(xmlSubNode->getNodeValue());
			osFam = atoi(sname);
			delete sname;
			delete tmpCh;
*/
			tmpCh = XMLString::transcode("name");
			xmlSubNode = xmlAtt->getNamedItem(tmpCh);
			sname = XMLString::transcode(xmlSubNode->getNodeValue());
			this->addFamily(sname);
			osFam = this->getFamily(sname);
			delete sname;
			delete tmpCh;

		} // for(XMLSize_t i = 0; i < nodesOSFam->getLength(); ++i)

		osFam = 0;
		sname = NULL;

		for(i = 0; i < nodesOSFamTex->getLength(); ++i)
		{
			node = nodesOSFamTex->item(i);
			xmlAtt = node->getAttributes();

			tmpCh = XMLString::transcode("texture_id");
			xmlSubNode = xmlAtt->getNamedItem(tmpCh);
			sname = XMLString::transcode(xmlSubNode->getNodeValue());
			osFam = atoi(sname);
			delete sname;
			delete tmpCh;

			tmpCh = XMLString::transcode("file");
			xmlSubNode = xmlAtt->getNamedItem(tmpCh);
			sname = XMLString::transcode(xmlSubNode->getNodeValue());
			m_TexStore->loadOSTexture(osFam, (m_DataDir + sname).c_str());
			delete sname;
			delete tmpCh;

		} // for(XMLSize_t i = 0; i < nodesOSFam->getLength(); ++i)

	} // if(m_Parser != NULL)

	return TRUE;
}

/*********************
	  OS Family
**********************/
void XMLConfig::addFamily(const char* family)
{
	m_OStoFamily[family] = m_FamIndex++;
}

int XMLConfig::getFamily(const char* family)
{
	return m_OStoFamily[family];
}
