/**************************************************************************
 *	Copyright (C) 2004  iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, 
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#if _MSC_VER > 1000
#pragma once
#endif

#ifndef _H_F3_XML_CFG
#define _H_F3_XML_CFG

#include "tex_cache.hpp"

#include <fstream>
#include <xercesc/dom/DOM.hpp>
#include <xercesc/parsers/XercesDOMParser.hpp>
#include <xercesc/sax/HandlerBase.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <iostream>

using namespace std;

XERCES_CPP_NAMESPACE_USE

class XMLConfig
{

public:
	XMLConfig();
	~XMLConfig();

	// Basic Parsing
	int initParser();
	bool parse(string file);
	bool parseOSInfo();
	bool parseSettings();
	
	// Element Parsing
	bool parseOSInfo(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode* node);
	bool parseDisplay(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode* node);
	bool parseScreenshot(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode* node);
	bool parseGraphics(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode* node);
	bool parseAudio(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode* node);


	// Settings
	bool loadSettings();
	bool saveSettings();
	
	// OS Info
	bool loadOSInfo();

	// OS Family
	void addFamily(const char* family);
	int getFamily(const char* family);

	// Texture Management (should be removed from xmlConfig)
	TextureStore* getTextureStore() { return m_TexStore; };

public: // Settings
	string m_ConfigFile;
	string m_OSInfoFile;
	string m_FontFile;

	TTF_Font  *m_Font;

	int m_ResW;
	int m_ResH;

	bool enableEarth;
	float m_RootTilt;
	string m_RootTex;

//Extensions
	bool enableVBO;
	bool enableCVA;


// Features
	bool enableLight;
	bool enableTex;
public:
	bool m_TreePorts;
	bool m_RotateOn;
	string m_DefaultFont;
	GLUtil* m_glUtil; 
	TextureStore* m_TexStore; // Decouple this
 	string m_DataDir;
private:
	map<string, int> m_OStoFamily;
	XERCES_CPP_NAMESPACE_QUALIFIER XercesDOMParser* m_Parser;
	ErrorHandler* m_errHandler;
	unsigned int m_FamIndex;
	bool inited;
};


#endif //_H_F3_XML_CFG
